// --------------------------------------------------------------------------------------
//
// RobStanze.CS                                                    Ulrich Mller 19.06.02
//                                                                 PCS1700       C# Final
// Beschicken der Stanzmaschine mit Transportband 51 663
// durch einen Industry Robot (Knickarm- oder Sulenrobot) im Simultanbetrieb
// frmMain      : MainThread Steuerung des Betiebsablaufs, Bedieneroberflche
// RobThread    : Thread     Beschickung durch Industry Robot
// StanzThread  : Thread     Betrieb der Stanzmaschine
// umFish30.DLL : Thread     Zugriff auf die Interfaces, durch MultiMediaTimer gesteuert
//
// Modell : 2 x Intelligent Interface an COM1 und COM2
//          alternativ : Intelligent Interface mit Extension Module, in diesem Fall
//          ist nur eine FishFace Instanz erforderlich,
//          die Modellkonstanten mssen entsprechend korrigiert werden
// Interface Belegung : ftS / COM1 Stanze, ftR / COM2 Industry Robot
// Zum Aufbau der Stanze siehe Stanze.HTM, Schaltung siehe Modellkonstanten
// Der Industry Robot (Sulen/Knickarm Robot) mu nach Handbuch aufgebuat sein.
// Schaltung siehe Modellkonstanten, End-/Impulstaster E1/2, 3/4, 5/6, 7/8
// Robot und Stanze sollten fest miteinander verbunden sein
//
// Der Bewegungsablauf des Robots wurde nur angedeutet und sollte nach Gegebenheiten
// und eigenen Vorstellungen ausgebaut werden. Dabei sollte bercksichtigt werden,
// dass alle vier Motoeren simultan laufen knnen. Bei einem Teilschritt knnen also
// mehrere ftR.SetMotor einem ftR.WaitForMotors vorangehen (siehe auch Home)
//
// --------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using FishFa30;

namespace RobStanze
{
	public class frmMain : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button cmdEnde;
		private System.Windows.Forms.Button cmdAction;

		// --- FishFace : Klassen-Instanz -----------------------------------------------

		private FishFace ftR = new FishFace(false, false, 0);
		private FishFace ftS = new FishFace(false, false, 0);
		private Thread   robThread;              
		private Thread   stanzThread;
		private AutoResetEvent robReady;
		private AutoResetEvent stanzeReady;

		// --- Modellkonstanten -------------------------------------------------------

		const int 
			mBand   = 1, cbVor   = 1, cbRuck   = 2,
			ePhotoV = 1, ePhotoS = 2, mLampen  = 3,
			mStanze = 2, eOben   = 3, eUnten   = 4, csAb      = 1, csAuf = 2,
			mSaule  = 1, mArmV   = 3, mArmH    = 2, mGreifer  = 4,
			ftiEin  = 1, ftiAus  = 0, ftiLinks = 1, ftiRechts = 2;
		private System.Windows.Forms.Label lblRobot;
		private System.Windows.Forms.Label lblStanze;

		private System.ComponentModel.Container components = null;

		// --- ftmMain : Konstruktor/Destruktor ----------------------------------------

		public frmMain()
		{
			InitializeComponent();
		}
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.lblRobot = new System.Windows.Forms.Label();
			this.lblStanze = new System.Windows.Forms.Label();
			this.cmdEnde = new System.Windows.Forms.Button();
			this.cmdAction = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// lblRobot
			// 
			this.lblRobot.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lblRobot.Location = new System.Drawing.Point(16, 24);
			this.lblRobot.Name = "lblRobot";
			this.lblRobot.Size = new System.Drawing.Size(320, 24);
			this.lblRobot.TabIndex = 0;
			this.lblRobot.Text = "RobThread";
			this.lblRobot.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// lblStanze
			// 
			this.lblStanze.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lblStanze.Location = new System.Drawing.Point(16, 64);
			this.lblStanze.Name = "lblStanze";
			this.lblStanze.Size = new System.Drawing.Size(320, 24);
			this.lblStanze.TabIndex = 1;
			this.lblStanze.Text = "StanzThread";
			this.lblStanze.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// cmdEnde
			// 
			this.cmdEnde.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.cmdEnde.Location = new System.Drawing.Point(224, 128);
			this.cmdEnde.Name = "cmdEnde";
			this.cmdEnde.Size = new System.Drawing.Size(112, 32);
			this.cmdEnde.TabIndex = 2;
			this.cmdEnde.Text = "&ENDE";
			this.cmdEnde.Click += new System.EventHandler(this.cmdEnde_Click);
			// 
			// cmdAction
			// 
			this.cmdAction.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.cmdAction.Location = new System.Drawing.Point(224, 176);
			this.cmdAction.Name = "cmdAction";
			this.cmdAction.Size = new System.Drawing.Size(112, 32);
			this.cmdAction.TabIndex = 3;
			this.cmdAction.Text = "&START";
			this.cmdAction.Click += new System.EventHandler(this.cmdAction_Click);
			// 
			// frmMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(354, 229);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.cmdAction,
																		  this.cmdEnde,
																		  this.lblStanze,
																		  this.lblRobot});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Name = "frmMain";
			this.Text = "RobStanze : Rob fttert Stanzmaschine 51 663";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.frmMain_Closing);
			this.ResumeLayout(false);

		}
		#endregion

		// --- Der Haupteinstiegspunkt fr die Anwendung -------------------------------

		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain
				());
		}

		
		// --- cmdEnde_Click : Beenden des Modellbetriebs -----------------------------
		//     bei Button = HALT, Beenden des Programms, bei Button = ENDE

		private void cmdEnde_Click(object sender, System.EventArgs e)
		{
			if(cmdEnde.Text == "&HALT")
			{
				// --- Beenden des Stanzbetriebes --------------------------------------
				lblStanze.Text = "Beenden wird eingeleitet";
				ftR.NotHalt = true;       // --- SchleifenEnde
				stanzeReady.Set();        // --- Fugngerphase schalten
				robThread.Join();         // --- Warten auf Ende AutoThread
				ftS.NotHalt = true;
				robReady.Set();
				stanzThread.Join();       // --- Warten auf Ende FuThread

				ftR.CloseInterface();      // --- Interface abschalten
				ftS.CloseInterface();

				cmdAction.Enabled = true; // --- Buttons sortieren
				cmdEnde.Text = "&ENDE";
			}
			else
			{
				// --- Programmende --------------------------------------------------
				this.Close();
			}
		}

		// --- Schlieen der Form ablehnen, wenn das Modell noch luft ---------------

		private void frmMain_Closing(object sender, 
			System.ComponentModel.CancelEventArgs e)
		{
			if(cmdEnde.Text == "&HALT") e.Cancel = true;
		}

		// --- cmdAction_Click : Starten des Stanzbetriebes --------------------------

		private void cmdAction_Click(object sender, System.EventArgs e)
		{
			try
			{
				ftR.OpenInterface(Port.COM2, false); // --- Verbindung zum Interface COM2
											  		 //     ohne DoEvents
				ftS.OpenInterface(Port.COM1, false);

				cmdAction.Enabled = false;      // --- Buttons sortieren
				cmdEnde.Text      = "&HALT";
				lblStanze.Text   = "Stanze luft";
				lblRobot.Text      = "Robot luft";
				cmdEnde.Focus();

				// --- Threads, Events instanzieren und starten -----------------------
				robThread = new Thread(new ThreadStart(robExecute));
				stanzThread = new Thread(new ThreadStart(stanzExecute));
				robReady    = new AutoResetEvent(false);
				stanzeReady = new AutoResetEvent(false);
				robThread.Start();
				stanzThread.Start();
			}
			catch(FishFaceException eft)
			{
				lblRobot.Text = eft.Message;
				return;
			}
		}

		// --- robExecute : Nutzroutine des RobThread ---------------------------

		private void robExecute()
		{
			// --- Home ---------------------------------------------------------
			ftR.SetMotor(mSaule,    ftiLinks, 15, 999);
			ftR.SetMotor(mArmV,     ftiLinks, 15, 999);
			ftR.SetMotor(mArmH,     ftiLinks, 15, 999);
			ftR.SetMotor(mGreifer,  ftiLinks, 15, 999);
			ftR.WaitForMotors(0, mSaule, mArmV, mArmH, mGreifer);
			ftR.SetMotor(mSaule, ftiRechts, 15, 10);
			ftR.WaitForMotors(0, mSaule);

			do
			{
				// --- Greifen -------------------------------------------------
				lblRobot.Text = "Greifen";
				ftR.SetMotor(mGreifer, ftiRechts, 15, 10);
				ftR.WaitForMotors(0, mGreifer);

				// --- Zur Ablage ---------------------------------------------
				lblRobot.Text = "Zur Ablage";
				ftR.SetMotor(mSaule, ftiRechts, 15, 100);
				ftR.WaitForMotors(0, mSaule);

				// --- Ablegen/Hoch -------------------------------------------
				stanzeReady.WaitOne();
				lblRobot.Text = "Ablegen";
				ftR.SetMotor(mGreifer, ftiLinks, 15, 999);
				ftR.WaitForMotors(0, mGreifer);
				robReady.Set();

				// --- Zum Magazin -------------------------------------------
				lblRobot.Text = "zum Magazin";
				ftR.SetMotor(mSaule, ftiLinks, 15, 100);
				ftR.WaitForMotors(0, mSaule);
			} while(!ftR.Finish());
		}
		
		// --- stanzExecute : Nutzroutine des StanzThreads --------------------

		private void stanzExecute()
		{
			ftS.ClearMotors();
			ftS.SetMotor(mStanze, csAuf);
			ftS.WaitForInput(eOben);
			ftS.SetMotor(mStanze, ftiAus);
			ftS.SetMotor(mLampen, ftiEin);
			do
			{
				// --- Wartet auf Rob  -----------------------------------------
				lblStanze.Text = "Wartet";
				stanzeReady.Set();
				robReady.WaitOne();
				ftS.WaitForLow(ePhotoV);        // --- Test, wenn der Rob nicht liefert
				ftS.Pause(1234);                //     mit Hand nachreichen.

				// --- Zur Stanze -----------------------------------------------
				lblStanze.Text = "Zur Stanze";
				ftS.SetMotor(mBand, cbVor);
				ftS.WaitForLow(ePhotoS);
				ftS.SetMotor(mBand, ftiAus);
				
				// --- Stanzen --------------------------------------------------
				lblStanze.Text = "Stanzen";
				ftS.SetMotor(mStanze, csAb);
				ftS.WaitForInput(eUnten);
				ftS.SetMotor(mStanze, ftiAus);
				ftS.Pause(1000);
				ftS.SetMotor(mStanze, csAuf);
				ftS.WaitForInput(eOben);
				ftS.SetMotor(mStanze, ftiAus);

				// --- Zurck ---------------------------------------------------
				lblStanze.Text = "Zurck";
				ftS.SetMotor(mBand, cbRuck);
				ftS.WaitForInput(ePhotoV, false);
				ftS.Pause(300);
				ftS.SetMotor(mBand, ftiAus);
			} while(!ftS.Finish());
		}
	}
}
